/*
* smartt-client.c
*
* System Metrics - Annotation Recording and Tracing Tool (SMARTT)  
*
* This is a client application that displays cpu-cycles, bus-cycles, instructions, 
* cache-misses, branches along with the player data such as FPS, Drop Rate etc
* The watemark levels for all the metrics can be set and are exclusively marked as
* watermark highligted regions.
*
* Sample output:
*
*[ PID(s) ][ TIME-STAMP ][ CPU-CYCLES (w) ][ CACHE-M'SS (w) ][ BRANCHES   (w) ]
*
* 3149      1298458764    5186403           3743842    (+)    1369802    (+)    
* 3149      1298458765    19817424   (+)    22701574          345118     (+)    
* 3149      1298458766    4727479           1447752    (+)    1168918        
*
* Copyright (C) 2010>

* Author(s):
*
*   Sudip Jain  <sudip.jain@linaro.org>
*               <sudip.jain@stericsson.com>
*
* Released under GPL v2
*/

#include "smartt-userconfig.c"
#include "smartt-network.c"
#include "smartt-utils.c"

#include "smartt-client.h"
#include "smartt-userconfig.h"
#include "smartt-network.h"
#include "smartt-utils.h"


struct s_client_handler client_handler;
struct s_report_manager report_manager;
struct s_system_data system_data;
//struct s_mm_data mm_data;
//struct s_mem_info mem_info;

int main() 
{

    char  line[1024];               /* the input line                 */
    char  *argv[64];                /* the command line argument      */

    SMARTTclient_Init();

    while (1)
    {                                       /* repeat until done ....         */
        fprintf(stderr,"\n[SMARTT-CLIENT] # ");       /*   display a prompt             */
        fgets(line, sizeof(line),stdin);    /*   read in the command line     */
        smartt_util_parse(line, argv);         /*   parse the line               */
        if (strcmp(argv[0],"run")==0 || strcmp(argv[0],"r")==0)
            SMARTTclient_Run();
        if (strcmp(argv[0],"configure")==0 || strcmp(argv[0],"conf")==0)
            SMARTTclient_SetUserConfig(&argv[1]);
        if (strcmp(argv[0],"stat")==0)
            SMARTTclient_SetUserPid(&argv[1]);
        if (strcmp(argv[0], "exit") == 0 || strcmp(argv[0], "x") == 0 || strcmp(argv[0], "q") == 0)
            exit(0);              /*   exit if it is                */
    }
}

void SMARTTclient_Run()
{

    char ack_msg[64];
    int status, n;

    fprintf(stderr, "\n");
    fprintf(stderr, "[I] SMARTT Client Started\t\t\t\t: Success\n");

    sprintf(client_handler.msg_id, "REQ:CONNECT");
    sendto(tx_sock_desc, (void *)&client_handler, sizeof(client_handler), 0,(struct sockaddr *)&Tx_server_addr, sizeof(Tx_server_addr));

    fprintf(stderr, "[I] Waiting SMARTT Server to Accept \t\t\t: Waiting\n");
    fflush(stdout);

    n = recvfrom(rx_sock_desc, ack_msg, sizeof(ack_msg), 0, (struct sockaddr *)&Rx_client_addr, &addr_len);    
    if (strcmp(ack_msg,"ACK:ACCEPT") == 0)
        fprintf(stderr, "[I] Accepted Connection from SMARTT Server\t\t: Success\n");
    else
        fprintf(stderr, "[I] Connection from SMARTT Server\t\t\t: Failed\n");

    status = SMARTTclient_FilterSystemEvents();

    fprintf(stderr, "[I] Enter the Pid to track (PID=0 for smartt-player)\t: ");
    scanf("%u", &g_usr_pid);
    if (g_usr_pid == 0)
        client_handler.pid = 0;
    else
        client_handler.pid = g_usr_pid;

    if (status == 1)
    {
        sprintf(client_handler.msg_id, "REQ:USER-CONFIG");
        sendto(tx_sock_desc, (void *)&client_handler, sizeof(client_handler),0,(struct sockaddr *) &Tx_server_addr, sizeof(Tx_server_addr));
    } else
    {
        fprintf(stderr, "[E] Error in user configuration\t\t: Failed\n");
    }

    n = recvfrom(rx_sock_desc, ack_msg, sizeof(ack_msg), 0, (struct sockaddr *)&Rx_client_addr, &addr_len);    
    if (strcmp(ack_msg,"ACK:USER-CONFIG") == 0)
        fprintf(stderr, "[I] Configuring SMARTT Server\t\t\t\t: Success\n");
    else
        fprintf(stderr, "[I] Configuring SMARTT Server\t\t\t\t: Failed\n");

    sleep(1);

    SMARTTclient_ShowDisplay();
    SMARTTclient_PrintHeaders();
    sendto(tx_sock_desc, "REQ:CAPTURE", 64, 0,(struct sockaddr *)&Tx_server_addr, sizeof(Tx_server_addr));
    SMARTTclient_PrintStats (client_handler.event_config.samples);
    SMARTTclient_GenerateReport();
}

void SMARTTclient_ShowDisplay()
{

    smartt_util_exec("clear");
    fprintf(stderr,"\n\t\t\t*************************************************************************************************************");
    fprintf(stderr,"\n\t\t\t*                                                                                                           *");
    fprintf(stderr,"\n\t\t\t*                           System Metrics - Annotation Recording & Tracing Tool (SMARTT)                   *");
    fprintf(stderr,"\n\t\t\t*                                                                                                           *");
    fprintf(stderr,"\n\t\t\t*************************************************************************************************************\n");
    fprintf(stderr,"\n");
}

void SMARTTclient_PrintHeaders()
{
    char TableCols[1024];

    TableCols[0] = '\0';
    strcat(TableCols,"[ PID(s) ][ TIME-STAMP ]");

    if (event_handler[0].event_enable)
        strcat(TableCols,"[  CPU-CYCLES   (w) ]");
    if (event_handler[1].event_enable)
        strcat(TableCols,"[  INSTRUCN'S   (w) ]");
    if (event_handler[2].event_enable)
        strcat(TableCols,"[  BUS-CYCLES   (w) ]");
    if (event_handler[3].event_enable)
        strcat(TableCols,"[  CACHE-M'SS   (w) ]");
    if (event_handler[4].event_enable)
        strcat(TableCols,"[  BRANCHES     (w) ]");
    if (event_handler[5].event_enable)
        strcat(TableCols,"[  MEM-USAGE    (w) ]");

    strcat(TableCols,"[  CU-FPS  ]");
    strcat(TableCols,"[  AV-FPS  ]");
    strcat(TableCols,"[  REND'D  ]");
    strcat(TableCols,"[  DROP'D  ]");

    fprintf(stderr, "%s\n",TableCols);
    fflush(stdout);
}


void SMARTTclient_PrintStats(unsigned int sample_counter)
{
    unsigned int n;
    char temp_str[64];
    char stats_str[1024];

    while (sample_counter)
    {
        n  = recvfrom(rx_sock_desc, &system_data, sizeof(system_data), 0, (struct sockaddr *)&Rx_client_addr, &addr_len);
        
        stats_str[0] = '\0';    
        fprintf(stderr,"\n");

        strcat(stats_str, ADD_SPACES_2);
        sprintf(temp_str,"%-6d",system_data.sys_info.pid);
        strcat(stats_str, temp_str);
        strcat(stats_str,ADD_SPACES_4);

        sprintf(temp_str,"%-10ld",system_data.sys_info.time_stamp);
        strcat(stats_str,temp_str);
        strcat(stats_str,ADD_SPACES_4);

        if (event_handler[0].event_enable)
        {
            if ( system_data.sys_info.cpu_cycles < event_handler[0].watermark_level)
            {
                sprintf(temp_str,"cc %-10ld (.)",system_data.sys_info.cpu_cycles);
                strcat(stats_str, temp_str);
                strcat(stats_str,ADD_SPACES_4);
            } else
            {
                sprintf(temp_str,"cc %-10ld (+)",system_data.sys_info.cpu_cycles);
                strcat(stats_str, temp_str);
                strcat(stats_str,ADD_SPACES_4);
            }
        }

        if (event_handler[1].event_enable)
        {
            if (system_data.sys_info.instructions < event_handler[1].watermark_level)
            {
                sprintf(temp_str,"in %-10ld (.)",system_data.sys_info.instructions);
                strcat(stats_str, temp_str);
                strcat(stats_str,ADD_SPACES_4);
            } else
            {
                sprintf(temp_str,"in %-10ld (+)",system_data.sys_info.instructions);
                strcat(stats_str, temp_str);
                strcat(stats_str,ADD_SPACES_4);
            }

        }

        if (event_handler[2].event_enable)
        {
            if (system_data.sys_info.bus_cycles < event_handler[2].watermark_level)
            {
                sprintf(temp_str,"bc %-10ld (.)",system_data.sys_info.bus_cycles);
                strcat(stats_str, temp_str);
                strcat(stats_str,ADD_SPACES_4);
            } else
            {
                sprintf(temp_str,"bc %-10ld (+)",system_data.sys_info.bus_cycles);
                strcat(stats_str, temp_str);
                strcat(stats_str,ADD_SPACES_4);
            }
        }

        if (event_handler[3].event_enable)
        {
            if (system_data.sys_info.cache_misses < event_handler[3].watermark_level)
            {
                sprintf(temp_str,"cm %-10ld (.)",system_data.sys_info.cache_misses);
                strcat(stats_str, temp_str);
                strcat(stats_str,ADD_SPACES_4);
            } else
            {
                sprintf(temp_str,"cm %-10ld (+)",system_data.sys_info.cache_misses);
                strcat(stats_str, temp_str);
                strcat(stats_str,ADD_SPACES_4);
            }
        }

        if (event_handler[4].event_enable)
        {
            if (system_data.sys_info.branches < event_handler[4].watermark_level)
            {
                sprintf(temp_str,"br %-10ld (.)",system_data.sys_info.branches);
                strcat(stats_str, temp_str);
                strcat(stats_str,ADD_SPACES_4);
            } else
            {
                sprintf(temp_str,"br %-10ld (+)",system_data.sys_info.branches);
                strcat(stats_str, temp_str);
                strcat(stats_str,ADD_SPACES_4);
            }
        }

        if (event_handler[5].event_enable)
        {
            if (system_data.mem_info.main_used < event_handler[5].watermark_level)
            {
                sprintf(temp_str,"mu %-10ld (.)",system_data.mem_info.main_used);
                strcat(stats_str, temp_str);
                strcat(stats_str,ADD_SPACES_4);
            } else
            {
                sprintf(temp_str,"mu %-10ld (+)",system_data.mem_info.main_used);
                strcat(stats_str, temp_str);
                strcat(stats_str,ADD_SPACES_4);
            }
        }

            sprintf(temp_str,"cf %-6.2lf",system_data.player_info.curfps);
            strcat(stats_str, temp_str);
            strcat(stats_str,ADD_SPACES_3);

            sprintf(temp_str,"af %-6.2lf",system_data.player_info.avgfps);
            strcat(stats_str, temp_str);
            strcat(stats_str,ADD_SPACES_4);

            sprintf(temp_str,"re %-6lu",system_data.player_info.renderred );
            strcat(stats_str, temp_str);
            strcat(stats_str,ADD_SPACES_3);

            sprintf(temp_str,"dr %-6lu",system_data.player_info.dropped);
            strcat(stats_str, temp_str);

        fprintf(stderr, "%s",stats_str);
        sendto(trace_sock, stats_str, strlen(stats_str), 0, (struct sockaddr *)&TRACE_server_addr, sizeof(struct sockaddr));
        fflush(stdout);  
        sample_counter--;
    }
}

void SMARTTclient_GenerateReport()
{

    report_manager.stat_counter     =   client_handler.event_config.stat_counter;
    report_manager.total_samples    =   client_handler.event_config.samples;

    fprintf(stderr,"\n ########################################### REPORT GENERATION #############################################");
    fprintf(stderr,"\n Report No    :  %04d         Events Enabled      :  %04d                                                   ", ++report_manager.report_counter, report_manager.events_enabled);
    fprintf(stderr,"\n Start Time   :               Total Samples       :  %04d                                                   ", report_manager.total_samples);
    fprintf(stderr,"\n End Time     :               Stat Counter(secs)  :  %04.2lf                                                ", report_manager.stat_counter);
    fprintf(stderr,"\n Total Time   :               Watermark levels    :                                                         ");
    fprintf(stderr,"\n ###########################################################################################################\n");
    fflush(stdout);
}

int SMARTTclient_FilterSystemEvents()
{
    int i;

    for (i=0;i < NO_OF_EVENTS; i++)
    {
        if (event_handler[i].event_enable == 1)
        {
            client_handler.event_config.stat_counter = event_handler[i].stat_counter;
            client_handler.event_config.samples = event_handler[i].sample_count;
            break;
        }
    }

    for (i = 0;i < NO_OF_EVENTS; i++)
    {
        client_handler.event_config.event_enable[i] = event_handler[i].event_enable;

        if (client_handler.event_config.event_enable[i] == 1)
        {
            report_manager.events_enabled++;
            if (client_handler.event_config.stat_counter > event_handler[i].stat_counter)
                client_handler.event_config.stat_counter = event_handler[i].stat_counter;

            if (client_handler.event_config.samples < event_handler[i].sample_count)
                client_handler.event_config.samples = event_handler[i].sample_count;
        }
    }

    return(1);
}

void SMARTTclient_Init()
{
    int i;

    smartt_util_exec("clear");

    for (i=0;i < NO_OF_EVENTS;i++)
    {
        event_handler[i].event_name = *(sys_events+i);
    }
    SMARTTclient_GetUserConfig();
    SMARTTclient_GetNetworkConfig();

    for (i=0;i < sizeof(local_host.host_ip) ;i++)
        client_handler.local_client.host_ip[i] =  local_host.host_ip[i];
        client_handler.local_client.port_no =  local_host.port_no;


    SMARTTclient_OpenLocalServer();
    SMARTTclient_OpenRemoteServer();
    SMARTTclient_OpenTracePort();
}


