/*
 * smartt-userconfig.c
 *
 * User Configuration
*/

#include "smartt-userconfig.h"
#include "smartt-utils.h"

struct s_event_handler event_handler[NO_OF_EVENTS]; 
unsigned int g_usr_pid;

int SMARTTclient_SetUserConfig(char** conf_args)
{
    int i, args_error = 0;
    char *event_name, *event_param;
    double d_counter;
    unsigned int event_id, event_enable;
    unsigned int s_counter, wm_level;

    event_name  = *conf_args++;
    event_param = *conf_args++;

    if (strcmp(event_name,"default")==0)
    {
        SMARTTclient_SetConfig(CPU_CYCLES, EVENT_ENABLE, 2.0, 25, 40000);
        SMARTTclient_SetConfig(INSTRUCTIONS, EVENT_ENABLE, 3.0, 25, 40000);
        SMARTTclient_SetConfig(CACHE_MISSES, EVENT_ENABLE, 2.0, 25, 40000);
        SMARTTclient_SetConfig(BRANCHES, EVENT_ENABLE, 2.0, 25, 40000);
        SMARTTclient_SetConfig(BUS_CYCLES, EVENT_ENABLE, 2.0, 25, 40000);
    } else
    {

        if ((strcmp(event_name,"cpu-cycles")==0) || (strcmp(event_name,"cc")==0))
            event_id = 0;
        else if ((strcmp(event_name,"instructions")==0) || (strcmp(event_name,"in")==0))
            event_id = 1;
        else if ((strcmp(event_name,"bus-cycles")==0) || (strcmp(event_name,"bc")==0))
            event_id = 2;
        else if ((strcmp(event_name,"cache-misses")==0) || (strcmp(event_name,"cm")==0))
            event_id = 3;
        else if ((strcmp(event_name,"branches")==0) || (strcmp(event_name,"br")==0))
            event_id = 4;
        else if (strcmp(event_name,"all")==0)
            event_id = 64;
        else
            args_error = -1;

        if (strcmp(event_param,"-e")==0)
        {
            sscanf(*conf_args,"%d", &event_enable);
            if ((event_id==0)||(event_id==1)||(event_id==2)||(event_id==3)||(event_id==4))
                event_handler[event_id].event_enable = event_enable;
            else if ((strcmp(event_name,"all")==0))
            {
                for (i=0;i<=NO_OF_EVENTS;i++)
                    event_handler[i].event_enable = event_enable;
            } else
                args_error = -1;
        }

        if (strcmp(event_param,"-d")==0)
        {
            sscanf(*conf_args,"%lf", &d_counter);
            if ((event_id==0) || (event_id==1)||(event_id==2)||(event_id==3)||(event_id==4))
                event_handler[event_id].stat_counter = d_counter;
            else if ((strcmp(event_name,"all")==0))
            {
                for (i=0;i<=NO_OF_EVENTS;i++)
                    event_handler[i].stat_counter = d_counter;
            } else
                args_error = -1;
        }

        if (strcmp(event_param,"-s")==0)
        {
            sscanf(*conf_args,"%d", &s_counter);
            if ((event_id==0) || (event_id==1)||(event_id==2)||(event_id==3)||(event_id==4))
                event_handler[event_id].sample_count = s_counter;
            else if ((strcmp(event_name,"all")==0))
            {
                for (i=0;i<=NO_OF_EVENTS;i++)
                    event_handler[i].sample_count = s_counter;
            } else
                args_error = -1;
        }
        if (strcmp(event_param,"-w")==0)
        {
            sscanf(*conf_args,"%d", &wm_level);
            if ((event_id==0) || (event_id==1)||(event_id==2)||(event_id==3)||(event_id==4))
                event_handler[event_id].watermark_level = wm_level;
            else if ((strcmp(event_name,"all")==0))
            {
                for (i=0;i<=NO_OF_EVENTS;i++)
                    event_handler[i].watermark_level = wm_level;
            } else
                args_error = -1;
        }
    }

    if (args_error == -1)
    {
        fprintf(stderr, "[E] Incorrect command argument(s)\t\t: Error\n");
        return(0);
    }

    SMARTTclient_SaveUserConfig();
    fprintf(stderr,"[I] Configuration file 'smartt-user.config' saved\t\t: Success\n");
    return(1);
}

void SMARTTclient_GetUserConfig()
{
    int i=0;
    char event_name[40];
    double d_counter;
    int event_flag, s_count, wm_level;

    FILE *f1 = fopen("/etc/smartt/smartt-client.conf","r");
    if (f1== NULL)
    {
        fprintf(stderr, "[E] Cannot open /etc/smartt/smartt-client.conf \t\t: Error\n");
        exit(1);
    }
    while (!feof(f1))
    {
        fscanf(f1,"%s %d %lf %d %d\n",event_name,&event_flag,&d_counter,&s_count, &wm_level);
        event_handler[i].event_enable       =   (bool)event_flag;
        event_handler[i].stat_counter       =   d_counter;
        event_handler[i].sample_count       =   s_count;
        event_handler[i].watermark_level    =   wm_level;
        i++;  
    }

    fclose(f1);
}

int SMARTTclient_SaveUserConfig()
{
    int i;

    FILE *f1 = fopen("/etc/smartt/smartt-client.conf", "w");
    if (f1== NULL)
    {
        fprintf(stderr, "[E] Cannot open /etc/smartt/smartt-client.conf file\t\t: Error\n");
        exit (1);
    }

    for (i= 0;i < NO_OF_EVENTS;i++)
    {
        fprintf(f1,"%s%4d%8.2f%6d%12d\n",event_handler[i].event_name, event_handler[i].event_enable,
                event_handler[i].stat_counter, event_handler[i].sample_count,
                event_handler[i].watermark_level); 
    }

    fclose(f1);

    return(1);
}

int SMARTTclient_SetUserPid(char** conf_args)
{
    char *cmdline_idf, *cmdline_opt;
    
    cmdline_idf      =   *conf_args++;
    cmdline_opt      =   *conf_args++;

    if (strcmp(cmdline_idf,"-p")==0)
    {
        if (strcmp(cmdline_opt,"player")==0)
            g_usr_pid  =   0;
        else if (strcmp(cmdline_opt,"user")==0)
            sscanf(*conf_args,"%d", &g_usr_pid);
    }
    else
    {
        fprintf(stderr, "[E] Incorrect command argument %s\t\t: Error\n",cmdline_idf);
        return(0);
    }

    fflush(stdout);
    return(1);



//FILE *f1 = fopen("smartt-pid.list", "r");
//    if (f1== NULL)
//{
//    fprintf(stderr, "[E] Can't open 'smartt-pid.list' file\t\t: Error\n");
//    return(1);
//}
//while (!feof(f1))
//{
//    fscanf(f1,"%d\n",&user_pid);
//    i++;  
//}
//
//fclose(f1);

}

void SMARTTclient_SetConfig(t_event_id event, t_event_enable evt_enable,
                            t_stat_counter d_counter, t_sample_count s_count,
                            t_watermark_level wm_level)
{
    unsigned char event_id;
    event_id = (t_uint8)event;
    event_handler[event_id].event_enable    = (bool)evt_enable;
    event_handler[event_id].stat_counter    = d_counter;
    event_handler[event_id].sample_count    = s_count;
    event_handler[event_id].watermark_level = wm_level;
}


