/*
* main.c
*
* Media Player - Gsteramer based media player to display metrics data
*
* This is a media player that displays avg-fps, curr fps, drop rate, renderred and
* dropped frame data at specified interval which is configurable. It also display 
* widht, height and Bitrate of file. Audio codec and video codec used for playback
* is also displayed. This complete metrics data is also sent to smart server for 
* tracing.
*
* Sample input:
* 
* media_player {complete path of video file} such as:
* media_player /home/user/video.mp4
*
* Sample output:
*
* Bitrate of file is 1206992 
* video_codec is MPEG-4 video 
* audio_codec is MPEG-1 layer 3 
* width x height is 1280 x 720
*
* Pid=3819 Rend=41 Dropped=1 Width=320 Height=240 Curfps=26.52 dps=0.00 avgfps=41.00
* Pid=3819 Rend=53 Dropped=1 Width=320 Height=240 Curfps=23.46 dps=0.00 avgfps=26.50
* Pid=3819 Rend=66 Dropped=1 Width=320 Height=240 Curfps=26.52 dps=0.00 avgfps=33.00
*       
* Copyright (C) 2011,  <-->@linaro.org>

* Developed by:
*
*   Vishal Raj  <vishal.raj@linaro.org>
*               <vishal.raj@stericsson.com>
*
* Released under <Linaro 2010>
*/

#include <gtk/gtk.h>
#include <string.h>

#include "media_player.h"

static char *uri_to_play;
static gint fps_update_interval=500;

int

main (int argc, char *argv[])
{
    gtk_init (&argc, &argv);

    player_open();
    player_init (&argc, &argv);

    if (argc > 2)
    {
	fps_update_interval = atoi(argv[1]);        
        if (strchr (argv[2], ':'))
            uri_to_play = g_strdup (argv[2]);
        else
            uri_to_play = g_strdup_printf ("file://%s", argv[2]);
    }
    else if (argc > 1)
    {
        if (strchr (argv[1], ':'))
            uri_to_play = g_strdup (argv[1]);
        else
            uri_to_play = g_strdup_printf ("file://%s", argv[1]);    
    }
    else
    {
	fprintf(stderr,"\nUsage: media_player [fps-update-interval] {video_file_path}\n");
	fprintf(stderr,"\n\tfps-update-interval : time-interval in Millisec to get metrics data \n");
	fprintf(stderr,"\tvideo_file_path : absolute path of video file e.g. /home/user/video.mp4\n");
	fprintf(stderr,"\n\nExample: media_player 500 /home/user/video.mp4\n\n");
	exit(0);
    }

    if (uri_to_play)
        player_play (uri_to_play,fps_update_interval);

    gtk_main ();

    g_free (uri_to_play);

    player_deinit ();

    return 0;
}
